#include "MeshLoader.h"
#include <algorithm>


std::vector<Node>& MeshLoader::GetNode() {
	return node;
}

std::vector<FiniteElement>& MeshLoader::GetFiniteElement() {
	return finite_el;
}

std::vector<BoundaryFiniteElement>& MeshLoader::GetBoundaryElement() {
	return boundary_el;
}


std::vector<int> MeshLoader::ElementByThreeNodes(int id1, int id2, int id3) {
	std::vector<int> result;
	auto a = finite_el.begin();

	while (a != finite_el.end()) {
		a = find_if(a, finite_el.end(), [id1, id2, id3](FiniteElement s) {
			return (find(s.ID_Nodes.begin(), s.ID_Nodes.end(), id1) != s.ID_Nodes.end() &&
				find(s.ID_Nodes.begin(), s.ID_Nodes.end(), id2) != s.ID_Nodes.end() &&
				find(s.ID_Nodes.begin(), s.ID_Nodes.end(), id3) != s.ID_Nodes.end());
			});
		if (a != finite_el.end()) {
			result.push_back(a->ID_FiniteElement);
			a++;
		}
	}
	return result;
}


std::vector<int> MeshLoader::ElementByTwoNodes(int id1, int id2) {
	std::vector<int> result;
	auto a = finite_el.begin();

	while (a != finite_el.end()) {
		a = find_if(a, finite_el.end(), [id1, id2](const FiniteElement& elem) { 
			return (find(elem.ID_Nodes.begin(), elem.ID_Nodes.end(), id1) != elem.ID_Nodes.end() &&
				find(elem.ID_Nodes.begin(), elem.ID_Nodes.end(), id2) != elem.ID_Nodes.end());
			});
		if (a != finite_el.end()) {
			result.push_back(a->ID_FiniteElement);
			a++;
		}
	}
	return result;
}


std::set<int> MeshLoader::BoundaryNodesByID(int id) {
	std::set<int> result;
	for_each(boundary_el.begin(), boundary_el.end(), [&](const auto& elem1) {
		if (elem1.ID_Boundary == id) {
			for_each(elem1.ID_Nodes.begin(), elem1.ID_Nodes.end(), [&](const auto& elem2) {

				result.insert((node.at(elem2 - 1)).ID_Node);
				});
		}
		});
	return result;
}


std::set<int> MeshLoader::FiniteElementsByArea(int id_border) {
	std::set<int> result;
	auto BoundaryElement = boundary_el.begin();
	while (BoundaryElement != boundary_el.end()) {
		BoundaryElement = find_if(BoundaryElement, boundary_el.end(), [&](auto& elem) {return (elem.ID_Boundary == id_border); });
		if (BoundaryElement != boundary_el.end()) {
			auto a = ElementByThreeNodes(BoundaryElement->ID_Nodes.at(0), BoundaryElement->ID_Nodes.at(1), BoundaryElement->ID_Nodes.at(2));
			for (auto& i : a) {
				result.insert(i);
			}
			++BoundaryElement;
		}
	}
	return result;
}


std::vector<int> MeshLoader::BoundaryElementsByID(int id) {
	std::vector<int> result;
	for_each(boundary_el.begin(), boundary_el.end(), [&](const auto& elem) { 
		if (elem.ID_Boundary == id) {
			result.push_back(elem.ID_Element);
		}
		});

	return result;
}


void MeshLoader::insert_middle() {
	std::unordered_set<Edge, EdgeHash> List_edge;
	for (auto& i : finite_el) {
		std::vector<int> a = i.ID_Nodes;
		for (auto elem1 = a.begin(); elem1 < a.end(); elem1++) {
			for (auto elem2 = elem1 + 1; elem2 < a.end(); elem2++) {
				Edge ed(*elem1, *elem2);
				auto el = List_edge.find(ed);
				if (el == List_edge.end()) {
					Node el;
					el.ID_Node = node.size() + 1;
					el.flag = false;
					Node center_node(el.ID_Node,
						std::array<double, 3>{((node.at(*elem1 - 1).Coordinate.at(0) + node.at(*elem2 - 1).Coordinate.at(0)) / 2),
						((node.at(*elem1 - 1).Coordinate.at(1) + node.at(*elem2 - 1).Coordinate.at(1)) / 2),
						((node.at(*elem1 - 1).Coordinate.at(2) + node.at(*elem2 - 1).Coordinate.at(2)) / 2)}, false);
					node.push_back(el);
					i.ID_Nodes.push_back(el.ID_Node);
					ed.middle_id = el.ID_Node;
					List_edge.insert(ed);
				}
				else {
					i.ID_Nodes.push_back(el->middle_id);
				}
			}
		}
	}
	for (auto& i : boundary_el) {
		std::vector<int> a = i.ID_Nodes;
		for (auto elem1 = a.begin(); elem1 < a.end(); elem1++) {
			for (auto elem2 = elem1 + 1; elem2 < a.end(); elem2++) {
				Edge ed(*elem1, *elem2);
				if (List_edge.find(ed) != List_edge.end()) {
					auto el = List_edge.find(ed);
					i.ID_Nodes.push_back(el->middle_id);
				}
			}
		}
	}

}


std::vector<std::set<int>> MeshLoader::Neighbor_Nodes() {
	std::vector<std::set<int>> result(node.size());
	for (auto& elem : finite_el) {
		for (auto elem1 = elem.ID_Nodes.begin(); elem1 < elem.ID_Nodes.end(); elem1++) {
			for (auto elem2 = elem1 + 1; elem2 < elem.ID_Nodes.end(); elem2++) {
				result[*elem1 - 1].insert(*elem2);
			}
		}
	}
	return result;
}


void MeshLoader::print_DataTypes() {
	std::cout << "Nodes:"<< std::endl << node << std::endl
		<< "Finite Elements:" << std::endl<< finite_el << std::endl
		<< "Boundary Finite Elements:" << std::endl <<boundary_el << std::endl;
}

